import {ColumnKey, Columns} from "./Column";
import {CurrentRowData} from "./AsyncManager";

/**
 * Column sort type.
 */
export enum SortType
{
	ASC = "asc",
	DESC = "desc",
}

/**
 * Column sort state.
 */
export interface SortState
{
	/**
	 * Sort type (ascending or descending).
	 */
	type: SortType;

	/**
	 * Sort order.
	 */
	order: number;
}

/**
 * Sort the given rows from the columns sort state.
 * @param rows Rows to sort.
 * @param columns Columns definition.
 * @param columnsSortState Columns current sort state.
 */
export function sortRows<CK extends ColumnKey>(rows: CurrentRowData<CK>[], columns: Columns<CK>, columnsSortState: Partial<Record<CK, SortState>>): CurrentRowData<CK>[]
{
	// Normalize value to undefined when rows are not loaded.
	if (!Array.isArray(rows)) return undefined;

	// Get the sort column states following their priority order.
	const orderedSortColumns = (Object.entries(columnsSortState) as [CK, SortState][]).sort((a, b) => (
		a[1].order - b[1].order
	));

	// Sort the given rows.
	return rows.toSorted((a, b) => {
		for (const [columnKey, columnSort] of orderedSortColumns)
		{
			// Get the current comparison result for the given cells' data.
			const comparison =
				(columnSort.type == SortType.DESC ? 1 : -1) *
				(columns[columnKey]?.compare ?? genericCompare)(a.cells[columnKey]?.data, b.cells[columnKey]?.data);

			if (comparison != 0)
				// If an order can be determined from the current comparison, returning it.
				return comparison;
		}

		// The comparisons always returned 0.
		return 0;
	});
}

/**
 * Generic comparison function for table data.
 * @param a First data to compare.
 * @param b Second data to compare.
 */
export function genericCompare(a: any, b: any): number
{
	if (typeof a == "number" && typeof a == typeof b)
	{ // Compare numbers.
		return b - a;
	}
	else
	{ // Compare data as strings.
		return String(b).localeCompare(String(a));
	}
}
